@@ -17,6 +17,7 @@ import testtools
 
 from os_vif.internal.command import ip as ip_lib
 from oslo_concurrency import processutils
+from six.moves import builtins
 
 from vif_plug_ovs import constants
 from vif_plug_ovs import exception
@@ -40,12 +41,13 @@ class LinuxNetTest(testtools.TestCase):
                                             check_exit_code=[0, 2, 254])
         mock_dev_exists.assert_has_calls([mock.call("br0")])
 
+    @mock.patch.object(linux_net, "_arp_filtering")
     @mock.patch.object(ip_lib, "set")
     @mock.patch.object(os.path, "exists", return_value=False)
     @mock.patch.object(processutils, "execute")
     @mock.patch.object(linux_net, "device_exists", return_value=False)
     def test_ensure_bridge_new_ipv4(self, mock_dev_exists, mock_execute,
-                                    mock_path_exists, mock_ip_set):
+                                    mock_path_exists, mock_ip_set, *args):
         linux_net.ensure_bridge("br0")
 
         calls = [
@@ -61,12 +63,13 @@ class LinuxNetTest(testtools.TestCase):
         mock_ip_set.assert_called_once_with('br0', state='up',
                                             check_exit_code=[0, 2, 254])
 
+    @mock.patch.object(linux_net, "_arp_filtering")
     @mock.patch.object(ip_lib, "set")
     @mock.patch.object(os.path, "exists", return_value=True)
     @mock.patch.object(processutils, "execute")
     @mock.patch.object(linux_net, "device_exists", return_value=False)
     def test_ensure_bridge_new_ipv6(self, mock_dev_exists, mock_execute,
-                                    mock_path_exists, mock_ip_set):
+                                    mock_path_exists, mock_ip_set, *args):
         linux_net.ensure_bridge("br0")
 
         calls = [
@@ -400,7 +403,9 @@ class LinuxNetTest(testtools.TestCase):
     @mock.patch('six.moves.builtins.open')
     @mock.patch.object(os.path, 'isfile')
     @mock.patch.object(os, 'listdir')
-    def test_get_representor_port(self, mock_listdir, mock_isfile, mock_open):
+    @mock.patch.object(linux_net, "get_function_by_ifname")
+    def test_get_representor_port(self, mock_get_function_by_ifname,
+                                  mock_listdir, mock_isfile, mock_open):
         mock_listdir.return_value = [
             'pf_ifname', 'rep_vf_1', 'rep_vf_2'
         ]
@@ -408,7 +413,14 @@ class LinuxNetTest(testtools.TestCase):
         mock_open.return_value.__enter__ = lambda s: s
         readline_mock = mock_open.return_value.readline
         readline_mock.side_effect = (
-            ['pf_sw_id', 'pf_sw_id', '1', 'pf_sw_id', 'pf0vf2'])
+              ['pf_sw_id', 'pf_sw_id', '1', 'pf_sw_id', 'pf0vf2'])
+        # PCI IDs mocked:
+        # PF0:    0000:0a:00.0
+        # PF0VF1: 0000:0a:02.1    PF0VF2: 0000:0a:02.2
+        mock_get_function_by_ifname.side_effect = (
+            [("0000:0a:00.0", True),
+             ("0000:0a:02.1", False),
+             ("0000:0a:02.2", False), ("0000:0a:00.0", True)])
         open_calls = (
             [mock.call('/sys/class/net/pf_ifname/phys_switch_id', 'r'),
              mock.call().readline(),
@@ -432,8 +444,10 @@ class LinuxNetTest(testtools.TestCase):
     @mock.patch('six.moves.builtins.open')
     @mock.patch.object(os.path, 'isfile')
     @mock.patch.object(os, 'listdir')
+    @mock.patch.object(linux_net, "get_function_by_ifname")
     def test_get_representor_port_2_pfs(
-            self, mock_listdir, mock_isfile, mock_open):
+            self, mock_get_function_by_ifname, mock_listdir, mock_isfile,
+            mock_open):
         mock_listdir.return_value = [
             'pf_ifname1', 'pf_ifname2', 'rep_pf1_vf_1', 'rep_pf1_vf_2',
             'rep_pf2_vf_1', 'rep_pf2_vf_2',
@@ -442,15 +456,29 @@ class LinuxNetTest(testtools.TestCase):
         mock_open.return_value.__enter__ = lambda s: s
         readline_mock = mock_open.return_value.readline
         readline_mock.side_effect = (
-            ['pf1_sw_id', 'pf1_sw_id', 'pf2_sw_id', '1', 'pf1_sw_id', '2'])
-        ifname = linux_net.get_representor_port('pf_ifname1', '2')
-        self.assertEqual('rep_pf1_vf_2', ifname)
+            ['pf_sw_id',
+             'pf_sw_id', 'VF1@PF1', 'pf_sw_id', 'vf2@pf1',
+             'pf_sw_id', 'pf2vf1', 'pf_sw_id', 'pf2vf2'])
+        # PCI IDs mocked:
+        # PF1:    0000:0a:00.1    PF2:    0000:0a:00.2
+        # PF1VF1: 0000:0a:02.1    PF1VF2: 0000:0a:02.2
+        # PF2VF1: 0000:0a:04.1    PF2VF2: 0000:0a:04.2
+        mock_get_function_by_ifname.side_effect = (
+            [("0000:0a:00.1", True), ("0000:0a:00.2", True),
+             ("0000:0a:02.1", False), ("0000:0a:00.2", True),
+             ("0000:0a:02.2", False), ("0000:0a:00.2", True),
+             ("0000:0a:04.1", False), ("0000:0a:00.2", True),
+             ("0000:0a:04.2", False), ("0000:0a:00.2", True)])
+        ifname = linux_net.get_representor_port('pf_ifname2', '2')
+        self.assertEqual('rep_pf2_vf_2', ifname)
 
     @mock.patch('six.moves.builtins.open')
     @mock.patch.object(os.path, 'isfile')
     @mock.patch.object(os, 'listdir')
+    @mock.patch.object(linux_net, "get_function_by_ifname")
     def test_get_representor_port_not_found(
-            self, mock_listdir, mock_isfile, mock_open):
+            self, mock_get_function_by_ifname, mock_listdir, mock_isfile,
+            mock_open):
         mock_listdir.return_value = [
             'pf_ifname', 'rep_vf_1', 'rep_vf_2'
         ]
@@ -459,6 +487,13 @@ class LinuxNetTest(testtools.TestCase):
         readline_mock = mock_open.return_value.readline
         readline_mock.side_effect = (
             ['pf_sw_id', 'pf_sw_id', '1', 'pf_sw_id', '2'])
+        # PCI IDs mocked:
+        # PF0:    0000:0a:00.0
+        # PF0VF1: 0000:0a:02.1    PF0VF2: 0000:0a:02.2
+        mock_get_function_by_ifname.side_effect = (
+            [("0000:0a:00.0", True),
+             ("0000:0a:02.1", False),
+             ("0000:0a:02.2", False)])
         self.assertRaises(
             exception.RepresentorNotFound,
             linux_net.get_representor_port,
@@ -467,8 +502,10 @@ class LinuxNetTest(testtools.TestCase):
     @mock.patch('six.moves.builtins.open')
     @mock.patch.object(os.path, 'isfile')
     @mock.patch.object(os, 'listdir')
+    @mock.patch.object(linux_net, "get_function_by_ifname")
     def test_get_representor_port_exception_io_error(
-            self, mock_listdir, mock_isfile, mock_open):
+            self, mock_get_function_by_ifname, mock_listdir, mock_isfile,
+            mock_open):
         mock_listdir.return_value = [
             'pf_ifname', 'rep_vf_1', 'rep_vf_2'
         ]
@@ -477,6 +514,13 @@ class LinuxNetTest(testtools.TestCase):
         readline_mock = mock_open.return_value.readline
         readline_mock.side_effect = (
             ['pf_sw_id', 'pf_sw_id', IOError(), 'pf_sw_id', '2'])
+        # PCI IDs mocked:
+        # PF0:    0000:0a:00.0
+        # PF0VF1: 0000:0a:02.1    PF0VF2: 0000:0a:02.2
+        mock_get_function_by_ifname.side_effect = (
+            [("0000:0a:00.0", True),
+             ("0000:0a:02.1", False),
+             ("0000:0a:02.2", False), ("0000:0a:00.0", True)])
         self.assertRaises(
             exception.RepresentorNotFound,
             linux_net.get_representor_port,
@@ -485,8 +529,10 @@ class LinuxNetTest(testtools.TestCase):
     @mock.patch('six.moves.builtins.open')
     @mock.patch.object(os.path, 'isfile')
     @mock.patch.object(os, 'listdir')
+    @mock.patch.object(linux_net, "get_function_by_ifname")
     def test_get_representor_port_exception_value_error(
-            self, mock_listdir, mock_isfile, mock_open):
+            self, mock_get_function_by_ifname, mock_listdir, mock_isfile,
+            mock_open):
         mock_listdir.return_value = [
             'pf_ifname', 'rep_vf_1', 'rep_vf_2'
         ]
@@ -495,6 +541,13 @@ class LinuxNetTest(testtools.TestCase):
         readline_mock = mock_open.return_value.readline
         readline_mock.side_effect = (
             ['pf_sw_id', 'pf_sw_id', '1', 'pf_sw_id', 'a'])
+        # PCI IDs mocked:
+        # PF0:    0000:0a:00.0
+        # PF0VF1: 0000:0a:02.1    PF0VF2: 0000:0a:02.2
+        mock_get_function_by_ifname.side_effect = (
+            [("0000:0a:00.0", True),
+             ("0000:0a:02.1", False),
+             ("0000:0a:02.2", False)])
         self.assertRaises(
             exception.RepresentorNotFound,
             linux_net.get_representor_port,
@@ -575,3 +628,16 @@ class LinuxNetTest(testtools.TestCase):
             linux_net.get_vf_num_by_pci_address,
             '0000:00:00.1'
         )
+
+    @mock.patch.object(os.path, 'exists', return_value=True)
+    @mock.patch.object(builtins, 'open')
+    def test__arp_filtering(self, mock_open, *args):
+        mock_open.side_effect = mock.mock_open(read_data=mock.Mock())
+        linux_net._arp_filtering("br0")
+
+        mock_open.assert_has_calls([
+            mock.call('/proc/sys/net/ipv4/conf/br0/arp_ignore', 'w'),
+            mock.call('/proc/sys/net/ipv4/conf/br0/arp_announce', 'w')])
+        mock_open.side_effect.return_value.write.assert_has_calls([
+            mock.call('1'),
+            mock.call('2')])
